@@ -1,4 +1,5 @@
 import asyncio
+import base64
 import socket
 import random
 import binascii
@@ -98,14 +99,14 @@ def b2hex(bts):
     return binascii.hexlify(bts).decode("ascii")
 
 
-def hex2b(hex):
+def hex2b(hex_str):
     """Produce bytes from the given hex string representation.
 
     :param hex: hex string
     :type hex: str
     :rtype: bytes
     """
-    return binascii.unhexlify(hex.encode("ascii"))
+    return binascii.unhexlify(hex_str.encode("ascii"))
 
 
 tohex = bytes.hex if sys.version_info >= (3, 5) else b2hex
@@ -117,6 +118,18 @@ fromhex = bytes.fromhex if sys.version_info >= (3, 5) else hex2b
 """
 
 
+def to_base64_str(bytes_input) -> str:
+    return base64.b64encode(bytes_input).decode('utf-8')
+
+
+def base64_to_bytes(str_input) -> bytes:
+    return base64.b64decode(str_input.encode('utf-8'))
+
+
+def byte_bool(boolv):
+    return b'\x01' if boolv else b'\x00'
+
+
 async def event_wait(event, timeout, loop=None):
     """Wait for the given event to be set or for the timeout to expire.
 
